/**************************************************************

util.c - misc.

This file is a part of sscsss -- Simple Single Cycle Sample Set Synthesizer.

Copyright (C) 2002 David Baker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <errno.h>
#include <limits.h>

#include "sscsss.h"
#include "proto.h"

/**************************************************************

aconvi	- ascii to integer - disallow non-empty suffix
	returns 0 for ok

**************************************************************/

int aconvi(char *str, int *i)
{
    int ret;
    char *buf;
    long l;

    errno = 0;
    l = strtol(str, &buf, 10);
    if ( errno == ERANGE )		ret = -1;
    else if ( strlen(buf) != 0 )	ret = -2;
    else if ( l > INT_MAX )		ret = -3;
    else if ( l < INT_MIN )		ret = -4;
    else				ret =  0;
    *i = ( int ) l;
    return ret;
}

/**************************************************************

aconvd	- ascii to double - disallow non-empty suffix
	returns 0 for ok

**************************************************************/

int aconvd(char *str, double *d)
{
    int ret;
    char *buf;

    *d = strtod(str, &buf);
    if ( errno == ERANGE )		ret = -1;
    else if ( strlen(buf) != 0 )	ret = -2;
    else				ret =  0;
    return ret;
}

/**************************************************************

round - round floating point number using just the standard libs

**************************************************************/

double round(double x)
{
    double whole, fraction;
    fraction = modf(x, &whole);
    if      ( x > 0.0 ) {
	if  ( fraction >=  0.5 ) { whole += 1.0; }
    }
    else if ( x < 0.0 ) {
	if  ( fraction <= -0.5 ) { whole -= 1.0; }
    }
    else {
	whole = 0.0;
    }
    return(whole);
}

/**************************************************************

chkendianess - big or little
	returns 1 for little, 0 for big
	used by cp_rev_bytes, below

**************************************************************/

int chkendianess()
{
    int i;
    char *ptr;

    i = 1;
    ptr = (char *) &i;
    return *(ptr);
}

/**************************************************************

cp_rev_bytes - copy reversed bytes - not overlapped

**************************************************************/

void cp_rev_bytes(M_P m, unsigned char from[], unsigned char to[], int n)
{
    static int endianess = -1;

    if ( endianess == -1 ) endianess = chkendianess();

    if ( endianess ) {
	if ( n == 2 ) {
	    to[0] = from[1];
	    to[1] = from[0];
	}
	else if ( n == 3 ) { /* this assumes copying from a 4 byte integer */
	    to[0] = from[2];
	    to[1] = from[1];
	    to[2] = from[0];
	}
	else if ( n == 4 ) {
	    to[0] = from[3];
	    to[1] = from[2];
	    to[2] = from[1];
	    to[3] = from[0];
	}
	else errex(m, 0, "cp_rev_bytes called with n != 2, 3, or 4.");
    }
    else {
	if ( n == 2 ) {
	    to[0] = from[0];
	    to[1] = from[1];
	}
	else if ( n == 3 ) { /* this assumes copying from a 4 byte integer */
	    to[0] = from[1];
	    to[1] = from[2];
	    to[2] = from[3];
	}
	else if ( n == 4 ) {
	    to[0] = from[0];
	    to[1] = from[1];
	    to[2] = from[2];
	    to[3] = from[3];
	}
	else errex(m, 0, "cp_rev_bytes called with n != 2, 3, or 4.");
    }
}

/* end of util.c */

